]> cloudbase.mooo.com Git - z180-stamp.git/blobdiff - avr/cli.c
Z180 parameter checks
[z180-stamp.git] / avr / cli.c
index 046859c8f1648eadf8af4afcd19730de3242e4a3..1a26206673bb12c54eae48c5328359e6f3da3a98 100644 (file)
--- a/avr/cli.c
+++ b/avr/cli.c
@@ -1,3 +1,17 @@
+/*
+ * (C) Copyright 2014-2016 Leo C. <erbl259-lmu@yahoo.de>
+ *
+ * (C) Copyright 2000
+ * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
+ *
+ * Add to readline cmdline-editing by
+ * (C) Copyright 2005
+ * JinHua Luo, GuangDong Linux Center, <luo.jinhua@gd-linux.com>
+ *
+ * SPDX-License-Identifier:    GPL-2.0
+ */
+
+#include "cli.h"
 #include "common.h"
 
 #include <string.h>
@@ -12,7 +26,9 @@
 #include "env.h"
 #include "cli_readline.h"
 #include "con-utils.h"
-#include "cli.h"
+
+
+/* FIXME: Quoting problems */
 
 #define DEBUG_PARSER   0       /* set to 1 to debug */
 
@@ -28,12 +44,10 @@ static int cli_parse_line(char *line, char *argv[])
 
        debug_parser("%s: \"%s\"\n", __func__, line);
 
-       for (outp = inp = line, quote = '\0' ;
-                       nargs < CONFIG_SYS_MAXARGS && (c = *inp) != '\0';
-                       inp++) {
+       for (outp = inp = line, quote = '\0'; (c = *inp) != '\0'; inp++) {
 
                switch (state) {
-               case 0:
+               case 0:                                         /* before arg string, waiting for arg start */
                        if (isblank(c))
                                continue;
 
@@ -42,7 +56,7 @@ static int cli_parse_line(char *line, char *argv[])
                        state = 1;
                        /* fall thru */
 
-               case 1:
+               case 1:                                         /* in arg string, waiting for end of arg string */
                        if (c == '\\') {
                                ++state;
                                continue;
@@ -58,7 +72,7 @@ static int cli_parse_line(char *line, char *argv[])
                        }
                        break;
 
-               case 3:
+               case 3:                                         /* in quote */
                        if (c == '\\' && quote == '\"') {
                                ++state;
                                continue;
@@ -69,12 +83,17 @@ static int cli_parse_line(char *line, char *argv[])
                        }
                        break;
 
-               case 2:
+               case 2:                                         /* waiting for next char */
                case 4:
                        --state;
                        break;
 
                }
+
+               if (nargs > CONFIG_SYS_MAXARGS) {
+                       --nargs;
+                       break;
+               }
                *outp++ = c;
        }
 
@@ -158,7 +177,7 @@ char *process_macros(char *input, char *output)
                                if (c == '}') {
                                        /* Terminate variable name */
                                        *(inp-1) = '\0';
-                                       const char *envval = getenv(varname);
+                                       const char *envval = getenv_char(varname);
                                        *(inp-1) = '}';
                                        /* Copy into the line if it exists */
                                        if (envval != NULL)
@@ -191,7 +210,7 @@ char *process_macros(char *input, char *output)
  * WARNING:
  *
  * We must create a temporary copy of the command since the command we get
* may be the result from getenv(), which returns a pointer directly to
  * the environment data, which may change magicly when the command we run
  * creates or modifies environment variables (like "bootp" does).
  *
@@ -239,25 +258,29 @@ static int cli_run_command(const char *cmd, int flag)
                 */
                for (inquotes = 0, sep = str; *sep; sep++) {
                        if ((*sep == '\'') &&
-                           (*(sep - 1) != '\\'))
+                           (sep != str) &&                                     /* past string start */
+                           (*(sep - 1) != '\\'))                       /* and NOT escaped */
                                inquotes = !inquotes;
 
                        if (!inquotes &&
-                           (*sep == ';' || *sep == '\n') &&    /* separator            */
-                           (sep != str) &&                     /* past string start    */
-                           (*(sep - 1) != '\\'))               /* and NOT escaped */
+                           (*sep == ';' || *sep == '\n'        /* separator */
+                                       || *sep == '#') &&                      /*   or start of comment */
+                           ((sep == str) ||                            /* string start */
+                           (*(sep - 1) != '\\')))                      /*   or NOT escaped */
                                break;
                }
 
-               /*
-                * Limit the token to data between separators
-                */
+               /* no more commands after unescaped '#' token */
+               if (*sep == '#')
+                       *sep = '\0';
+
+               /* Limit the token to data between separators */
                token = str;
                if (*sep) {
-                       str = sep + 1;  /* start of command for next pass */
+                       str = sep + 1;          /* start of command for next pass */
                        *sep = '\0';
                } else {
-                       str = sep;      /* no more commands for next pass */
+                       str = sep;                      /* no more commands for next pass */
                }
                debug_parser("token: \"%s\"\n", token);
 
@@ -296,7 +319,7 @@ static int cli_run_command_list(const char *cmd)
 
 
 /*
- * Run a command using the selected parser.
+ * Run a command.
  *
  * @param cmd  Command to run
  * @param flag Execution flags (CMD_FLAG_...)
@@ -344,7 +367,7 @@ void cli_loop(void)
        int rc = 1;
 
        for (;;) {
-               len = cli_readline(PSTR(CONFIG_SYS_PROMPT));
+               len = cli_readline(PSTR(CONFIG_SYS_PROMPT), 1);
 
                flag = 0;       /* assume no special flags for now */
                if (len > 0) {
@@ -365,27 +388,3 @@ void cli_loop(void)
                }
        }
 }
-
-
-command_ret_t do_run(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
-{
-       int i;
-       (void) cmdtp;
-
-       if (argc < 2)
-               return CMD_RET_USAGE;
-
-       for (i = 1; i < argc; ++i) {
-               char *arg;
-
-               arg = getenv(argv[i]);
-               if (arg == NULL) {
-                       printf_P(PSTR("## Error: \"%s\" not defined\n"), argv[i]);
-                       return CMD_RET_FAILURE;
-               }
-
-               if (run_command(arg, flag) != 0)
-                       return CMD_RET_FAILURE;
-       }
-       return CMD_RET_SUCCESS;
-}
@@ -191,7 +210,7 @@ char *process_macros(char *input, char *output)
  *
  * We must create a temporary copy of the command since the command we get
- * may be the result from getenv(), which returns a pointer directly to
+ * may be the result from getenv_char(), which returns a pointer directly to
  * the environment data, which may change magicly when the command we run
  * creates or modifies environment variables (like "bootp" does).